// Sample code: Using the GPU/CUDA backend extract facial feature vectors for a batch of face chips.
// Build on Linux: g++ -o batch_fr_cuda --std=c++11 ./batch_fr_cuda.cpp -I ../ -Wl,-rpath ../trueface_sdk ../trueface_sdk/libtf.so -fopenmp

#include "trueface_sdk/tf_sdk.h"
#include <iostream>
#include <vector>

int main() {

    Trueface::ConfigurationOptions options;
    options.frModel = Trueface::FacialRecognitionModel::FULL;
    options.enableGPU = true;

    Trueface::SDK tfSdk(options);

    bool valid = tfSdk.setLicense("${LICENSE_CODE}");

    if (valid == false) {
        std::cout << "Error: the provided license is invalid." << std::endl;
        return 1;
    }

    std::vector<uint8_t*> vecFaceChips;
    uint8_t faceChips[3][112][112][3];

    for (int i=0; i<3; i++) {
        Trueface::ErrorCode errorCode = tfSdk.setImage("../images/armstrong_"+std::to_string(i+1)+".jpg");
        if (errorCode != Trueface::ErrorCode::NO_ERROR) {
            std::cout<<"Error: could not load the image"<<std::endl;
            return 1;
        }

        Trueface::FaceBoxAndLandmarks faceBoxAndLandmarks;
        bool found = false;
        tfSdk.detectLargestFace(faceBoxAndLandmarks, found);
        if (found == false) {
            std::cout<<"Error: could not detect a face"<<std::endl;
            return 1;
        }

        tfSdk.extractAlignedFace(faceBoxAndLandmarks, &(faceChips[i][0][0][0]));
        vecFaceChips.push_back(&(faceChips[i][0][0][0]));
    }

    std::vector<Trueface::Faceprint> faceprints;
    tfSdk.getFaceFeatureVectors(vecFaceChips, faceprints);

    float similarity;
    float probability;

    tfSdk.getSimilarity(faceprints[0], faceprints[1], probability, similarity);
    std::cout<<"1st face - 2nd face, match probability: "<<probability
             <<" cosine similarity: "<<similarity<<std::endl;     

    tfSdk.getSimilarity(faceprints[1], faceprints[2], probability, similarity);
    std::cout<<"2nd face - 3rd face, match probability: "<<probability
             <<" cosine similarity: "<<similarity<<std::endl;     


    return 0;
}
